@@ -7,11 +7,13 @@ use Getopt::Long;
 
 my $WIDTH    = 1024;
 my $HEIGHT   = 768;
+my $QUALITY  = 100;
 my $OUT_FILE = '';
 Getopt::Long::GetOptions(
-    'width=i'  => \$WIDTH,
-    'height=i' => \$HEIGHT,
-    'out=s'    => \$OUT_FILE,
+    'width=i'   => \$WIDTH,
+    'height=i'  => \$HEIGHT,
+    'quality=i' => \$QUALITY,
+    'out=s'     => \$OUT_FILE,
 );
 die "Need --out [file]\n" unless $OUT_FILE;
 
@@ -23,6 +25,7 @@ $webio->register( 'rpi', $rpi );
 
 $webio->img_set_width( 'rpi', 0, $WIDTH );
 $webio->img_set_height( 'rpi', 0, $HEIGHT );
+$webio->img_set_quality( 'rpi', 0, $QUALITY );
 
 open( my $out, '>', $OUT_FILE ) or die "Can't open '$OUT_FILE': $!\n";
 my $in = $webio->img_stream( 'rpi', 0, 'image/jpeg' );
