@@ -22,7 +22,7 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 # POSSIBILITY OF SUCH DAMAGE.
 package Device::WebIO::RaspberryPi;
-$Device::WebIO::RaspberryPi::VERSION = '0.006';
+$Device::WebIO::RaspberryPi::VERSION = '0.007';
 # ABSTRACT: Device::WebIO implementation for the Rapsberry Pi
 use v5.12;
 use Moo;
@@ -31,6 +31,7 @@ use HiPi::Wiring qw( :wiring );
 use HiPi::Device::I2C;
 use GStreamer1;
 use Glib qw( TRUE FALSE );
+use AnyEvent;
 
 use constant {
     TYPE_REV1         => 0,
@@ -107,6 +108,22 @@ use constant {
     },
 };
 
+my %ALLOWED_VIDEO_TYPES = (
+    'video/H264'      => 1,
+    'video/x-msvideo' => 1,
+
+    # mp4mux doesn't seem to like the stream-format that comes out of rpicamsrc.
+    # Converting might be too slow on the Rpi.  For reference, try to get this 
+    # pipeline to work (which won't link up as written):
+    #
+    # gst-launch-1.0 -v rpicamsrc ! h264parse ! \
+    #     'video/x-h264,width=800,height=600,fps=30,stream-format=avc' ! \
+    #     mp4mux ! filesink location=/tmp/output.mp4
+    #
+#    'video/mp4'       => 1,
+);
+
+
 has 'pin_desc', is => 'ro';
 has '_type',    is => 'ro';
 has '_pin_mode' => (
@@ -436,6 +453,218 @@ sub _get_i2c_device_by_channel
 }
 
 
+has '_vid_width' => (
+    is      => 'rw',
+    default => sub {[
+        1920
+    ]},
+);
+has '_vid_height' => (
+    is      => 'rw',
+    default => sub {[
+       1080 
+    ]},
+);
+has '_vid_fps' => (
+    is      => 'rw',
+    default => sub {[
+       30
+    ]},
+);
+has '_vid_bitrate' => (
+    is      => 'rw',
+    default => sub {[
+       8000
+    ]},
+);
+has '_vid_stream_callbacks' => (
+    is      => 'rw',
+    default => sub {[]},
+);
+has '_vid_stream_callback_types' => (
+    is      => 'rw',
+    default => sub {[]},
+);
+has 'cv' => (
+    is      => 'rw',
+    default => sub { AnyEvent->condvar },
+);
+with 'Device::WebIO::Device::VideoOutputCallback';
+
+sub vid_channels
+{
+    return 1;
+}
+
+sub vid_height
+{
+    my ($self, $pin) = @_;
+    return $self->_vid_height->[$pin];
+}
+
+sub vid_width
+{
+    my ($self, $pin) = @_;
+    return $self->_vid_width->[$pin];
+}
+
+sub vid_fps
+{
+    my ($self, $pin) = @_;
+    return $self->_vid_fps->[$pin];
+}
+
+sub vid_kbps
+{
+    my ($self, $pin) = @_;
+    return $self->_vid_bitrate->[$pin];
+}
+
+sub vid_set_width
+{
+    my ($self, $pin, $val) = @_;
+    return $self->_vid_width->[$pin] = $val;
+}
+
+sub vid_set_height
+{
+    my ($self, $pin, $val) = @_;
+    return $self->_vid_height->[$pin] = $val;
+}
+
+sub vid_set_fps
+{
+    my ($self, $pin, $val) = @_;
+    return $self->_vid_fps->[$pin] = $val;
+}
+
+sub vid_set_kbps
+{
+    my ($self, $pin, $val) = @_;
+    $val *= 1024;
+    return $self->_vid_bitrate->[$pin] = $val;
+}
+
+sub vid_allowed_content_types
+{
+    return keys %ALLOWED_VIDEO_TYPES;
+}
+
+sub vid_stream
+{
+    my ($self, $pin, $type) = @_;
+    die "Do not support type '$type'" unless exists $ALLOWED_VIDEO_TYPES{$type};
+    $self->_init_gstreamer;
+    return 1;
+}
+
+sub vid_stream_callback
+{
+    my ($self, $pin, $type, $callback) = @_;
+    die "Do not support type '$type'" unless exists $ALLOWED_VIDEO_TYPES{$type};
+    $self->_vid_stream_callbacks->[$pin] = $callback;
+    $self->_vid_stream_callback_types->[$pin] = $type;
+    return 1;
+}
+
+sub vid_stream_begin_loop
+{
+    my ($self, $channel) = @_;
+    my $width    = $self->vid_width( $channel );
+    my $height   = $self->vid_height( $channel );
+    my $fps      = $self->vid_fps( $channel );
+    my $bitrate  = $self->vid_kbps( $channel );
+    my $callback = $self->_vid_stream_callbacks->[$channel];
+    my $type     = $self->_vid_stream_callback_types->[$channel];
+
+
+    $self->_init_gstreamer;
+    my $cv = $self->cv;
+    my $pipeline = GStreamer1::Pipeline->new( 'pipeline' );
+
+    my $rpi        = GStreamer1::ElementFactory::make( rpicamsrc => 'and_who' );
+    my $h264parse  = GStreamer1::ElementFactory::make( h264parse => 'are_you' );
+    my $capsfilter = GStreamer1::ElementFactory::make(
+        capsfilter => 'the_proud_lord_said' );
+    my $sink    = GStreamer1::ElementFactory::make(
+        fakesink => 'that_i_should_bow_so_low' );
+
+    my $muxer = ($type ne 'video/H264')
+        ? $self->_get_vid_mux_by_type( $type )
+        : undef;
+
+    $rpi->set( bitrate => $bitrate );
+
+    my $caps = GStreamer1::Caps::Simple->new( 'video/x-h264',
+        width  => 'Glib::Int' => $width,
+        height => 'Glib::Int' => $height,
+        fps    => 'Glib::Int' => $fps,
+    );
+    $capsfilter->set( caps => $caps );
+
+    $sink->set( 'signal-handoffs' => TRUE );
+    $sink->signal_connect(
+        'handoff' => $self->_get_vid_stream_callback( $pipeline, $cv, $callback )
+    );
+
+    my @link = ( $rpi, $h264parse, $capsfilter );
+    push @link, $muxer if defined $muxer;
+    push @link, $sink;
+    $pipeline->add( $_ ) for @link;
+    foreach my $i (0 .. ($#link - 1)) {
+        my $this = $link[$i];
+        my $next = $link[$i+1];
+        $this->link( $next );
+    }
+
+    $pipeline->set_state( "playing" );
+    $cv->recv;
+    $pipeline->set_state( "null" );
+
+    return 1;
+}
+
+
+sub _get_vid_stream_callback
+{
+    my ($self, $pipeline, $cv, $callback) = @_;
+
+    my $full_callback = sub {
+        my ($sink, $data_buf, $pad) = @_;
+        my $size = $data_buf->get_size;
+        my $buf  = $data_buf->extract_dup( 0, $size, undef, $size );
+
+        $callback->( $buf );
+
+        return 1;
+    };
+
+    return $full_callback;
+}
+
+my %MUXER_BY_TYPE = (
+    'video/x-msvideo' => [
+        'avimux', {},
+    ],
+#    'video/mp4'       => [
+#        'mp4mux', {
+#            streamable => TRUE,
+#        },
+#    ],
+);
+sub _get_vid_mux_by_type
+{
+    my ($self, $type) = @_;
+    my ($muxer_name, $properties) = @{ $MUXER_BY_TYPE{$type} };
+    my $muxer = GStreamer1::ElementFactory::make( $muxer_name => 'muxer' );
+
+    for (keys %$properties) {
+        $muxer->set( $_ => $properties->{$_} );
+    }
+
+    return $muxer;
+}
+
 sub _pin_desc_rev1
 {
     return [qw{
@@ -513,11 +742,11 @@ sub _init_gstreamer
 }
 
 
+
 # TODO
 #with 'Device::WebIO::Device::SPI';
 #with 'Device::WebIO::Device::I2C';
 #with 'Device::WebIO::Device::Serial';
-#with 'Device::WebIO::Device::VideoStream';
 
 1;
 __END__
@@ -569,8 +798,19 @@ https://github.com/thaytan/gst-rpicamsrc
 
 =item * I2CProvider
 
+=item * VideoOutputCallback
+
 =back
 
+=head1 ADDITONAL METHODS
+
+=head2 cv
+
+Returns the condvar from C<AnyEvent>, which is used for video processing.  If 
+there's any events you would like to handle in between video frames, get this 
+condvar and use it with C<AnyEvent>. You may also use this method as a 
+setter before calling C<vid_stream_begin_loop()>.
+
 =head1 LICENSE
 
 Copyright (c) 2014  Timm Murray
